@@ -1,3 +1,4 @@
+# encoding: utf-8
 
 #
 # The MIT License
@@ -40,20 +41,20 @@ import os
 import time
 import locale
 import signal
-from types import ModuleType
 from optparse import Option
 from six.moves import range
 from six import iteritems, string_types
 
 from pygments.token import Token
 
-from bpython import args as bpargs, repl, translations
-from bpython._py3compat import py3
-from bpython.formatter import theme_map
-from bpython.importcompletion import find_coroutine
-from bpython.translations import _
+from . import args as bpargs, repl, translations
+from ._py3compat import py3
+from .config import getpreferredencoding
+from .formatter import theme_map
+from .importcompletion import find_coroutine
+from .translations import _
 
-from bpython.keys import urwid_key_dispatch as key_dispatch
+from .keys import urwid_key_dispatch as key_dispatch
 
 import urwid
 
@@ -72,19 +73,15 @@ Parenthesis = Token.Punctuation.Parenthesis
 
 COLORMAP = {
     'k': 'black',
-    'r': 'dark red', # or light red?
-    'g': 'dark green', # or light green?
+    'r': 'dark red',  # or light red?
+    'g': 'dark green',  # or light green?
     'y': 'yellow',
-    'b': 'dark blue', # or light blue?
-    'm': 'dark magenta', # or light magenta?
-    'c': 'dark cyan', # or light cyan?
+    'b': 'dark blue',  # or light blue?
+    'm': 'dark magenta',  # or light magenta?
+    'c': 'dark cyan',  # or light cyan?
     'w': 'white',
     'd': 'default',
-    }
-
-
-def getpreferredencoding():
-    return locale.getpreferredencoding() or "ascii"
+}
 
 
 try:
@@ -107,7 +104,6 @@ else:
             self.repl.main_loop.process_input(line)
             self.repl.main_loop.process_input(['enter'])
 
-
     class EvalFactory(protocol.ServerFactory):
 
         def __init__(self, myrepl):
@@ -145,7 +141,6 @@ if urwid.VERSION < (1, 0, 0) and hasattr(urwid, 'TwistedEventLoop'):
                 except:
                     # This is the same as in urwid.
                     # We are obviously not supposed to ever hit this.
-                    import sys
                     print(sys.exc_info())
                     self._exc_info = sys.exc_info()
                     self.reactor.crash()
@@ -174,8 +169,10 @@ class StatusbarEdit(urwid.Edit):
         else:
             return urwid.Edit.keypress(self, size, key)
 
+
 urwid.register_signal(StatusbarEdit, 'prompt_enter')
 
+
 class Statusbar(object):
 
     """Statusbar object, ripped off from bpython.cli.
@@ -233,7 +230,7 @@ class Statusbar(object):
 
     def prompt(self, s=None, single=False):
         """Prompt the user for some input (with the optional prompt 's'). After
-        the user hit enter the signal 'prompt_result' will be emited and the
+        the user hit enter the signal 'prompt_result' will be emitted and the
         status bar will be reset. If single is True, the first keypress will be
         returned."""
 
@@ -264,7 +261,7 @@ class Statusbar(object):
 
         self.text.set_text(('main', s))
         if permanent:
-          self.s = s
+            self.s = s
 
     def clear(self):
         """Clear the status bar."""
@@ -276,6 +273,7 @@ class Statusbar(object):
         self.settext(self.s)
         urwid.emit_signal(self, 'prompt_result', new_text)
 
+
 urwid.register_signal(Statusbar, 'prompt_result')
 
 
@@ -291,6 +289,7 @@ def decoding_input_filter(keys, raw):
             converted_keys.append(key)
     return converted_keys
 
+
 def format_tokens(tokensource):
     for token, text in tokensource:
         if text == '\n':
@@ -453,15 +452,18 @@ class BPythonEdit(urwid.Edit):
         finally:
             self._bpy_may_move_cursor = False
 
+
 class BPythonListBox(urwid.ListBox):
     """Like `urwid.ListBox`, except that it does not eat up and
     down keys.
     """
+
     def keypress(self, size, key):
         if key not in ["up", "down"]:
             return urwid.ListBox.keypress(self, size, key)
         return key
 
+
 class Tooltip(urwid.BoxWidget):
 
     """Container inspired by Overlay to position our tooltip.
@@ -543,6 +545,7 @@ class Tooltip(urwid.BoxWidget):
         canvas.cursor = cursor
         return canvas
 
+
 class URWIDInteraction(repl.Interaction):
     def __init__(self, config, statusbar, frame):
         repl.Interaction.__init__(self, config, statusbar)
@@ -585,7 +588,7 @@ class URWIDInteraction(repl.Interaction):
 
 class URWIDRepl(repl.Repl):
 
-    _time_between_redraws = .05 # seconds
+    _time_between_redraws = .05  # seconds
 
     def __init__(self, event_loop, palette, interpreter, config):
         repl.Repl.__init__(self, interpreter, config)
@@ -599,7 +602,7 @@ class URWIDRepl(repl.Repl):
         self.tooltip = urwid.ListBox(urwid.SimpleListWalker([]))
         self.tooltip.grid = None
         self.overlay = Tooltip(self.listbox, self.tooltip)
-        self.stdout_hist = ''
+        self.stdout_hist = ''  # native str (bytes in Py2, unicode in Py3)
 
         self.frame = urwid.Frame(self.overlay)
 
@@ -616,12 +619,13 @@ class URWIDRepl(repl.Repl):
 
         # String is straight from bpython.cli
         self.statusbar = Statusbar(config,
-            _(" <%s> Rewind  <%s> Save  <%s> Pastebin "
-              " <%s> Pager  <%s> Show Source ") %
-              (config.undo_key, config.save_key, config.pastebin_key,
-               config.last_output_key, config.show_source_key), self.main_loop)
+                                   _(" <%s> Rewind  <%s> Save  <%s> Pastebin "
+                                     " <%s> Pager  <%s> Show Source ") %
+                                   (config.undo_key, config.save_key, config.pastebin_key,
+                                       config.last_output_key, config.show_source_key), self.main_loop)
         self.frame.set_footer(self.statusbar.widget)
-        self.interact = URWIDInteraction(self.config, self.statusbar, self.frame)
+        self.interact = URWIDInteraction(
+            self.config, self.statusbar, self.frame)
 
         self.edits = []
         self.edit = None
@@ -695,10 +699,11 @@ class URWIDRepl(repl.Repl):
         if self.edit is None:
             return ''
         return self.edit.get_edit_text()
+
     def _set_current_line(self, line):
         self.edit.set_edit_text(line)
     current_line = property(_get_current_line, _set_current_line, None,
-        "Return the current line (the one the cursor is in).")
+                            "Return the current line (the one the cursor is in).")
 
     def cw(self):
         """Return the current word (incomplete word left of cursor)."""
@@ -713,7 +718,7 @@ class URWIDRepl(repl.Repl):
 
         # Stolen from cli. TODO: clean up and split out.
         if (not text or
-            (not text[-1].isalnum() and text[-1] not in ('.', '_'))):
+                (not text[-1].isalnum() and text[-1] not in ('.', '_'))):
             return
 
         # Seek backwards in text for the first non-identifier char:
@@ -734,10 +739,11 @@ class URWIDRepl(repl.Repl):
 
     def _get_cursor_offset(self):
         return self.edit.edit_pos
+
     def _set_cursor_offset(self, offset):
         self.edit.edit_pos = offset
     cursor_offset = property(_get_cursor_offset, _set_cursor_offset, None,
-        "The cursor offset from the beginning of the line")
+                             "The cursor offset from the beginning of the line")
 
     def _populate_completion(self):
         widget_list = self.tooltip.body
@@ -811,7 +817,7 @@ class URWIDRepl(repl.Repl):
                                        (color, arg)])
                         if arg in kwonly_defaults:
                             markup.extend([('punctuation', '='),
-                                           ('token', kwonly_defaults[arg])])
+                                           ('token', repr(kwonly_defaults[arg]))])
 
                 if varkw:
                     if args or varargs or kwonly:
@@ -880,7 +886,8 @@ class URWIDRepl(repl.Repl):
             if py3:
                 self.stdout_hist += line + '\n'
             else:
-                self.stdout_hist += line.encode(locale.getpreferredencoding()) + '\n'
+                self.stdout_hist += line.encode(
+                    locale.getpreferredencoding()) + '\n'
             self.print_line(line)
             self.s_hist[-1] += self.f_string
             # I decided it was easier to just do this manually
@@ -901,7 +908,7 @@ class URWIDRepl(repl.Repl):
 
         self.evaluating = False
         #map(self.push, self.history)
-        #^-- That's how simple this method was at first :(
+        # ^-- That's how simple this method was at first :(
 
     def write(self, s):
         """For overriding stdout defaults"""
@@ -925,7 +932,6 @@ class URWIDRepl(repl.Repl):
         self.echo(s)
         self.s_hist.append(s.rstrip())
 
-
     def push(self, s, insert_into_history=True):
         # Restore the original SIGINT handler. This is needed to be able
         # to break out of infinite loops. If the interpreter itself
@@ -979,17 +985,17 @@ class URWIDRepl(repl.Repl):
         # input to be the same type, using ascii as encoding. If the
         # caption is bytes this breaks typing non-ascii into bpython.
         if not more:
+            caption = ('prompt', self.ps1)
             if py3:
-                caption = ('prompt', self.ps1)
+                self.stdout_hist += self.ps1
             else:
-                caption = ('prompt', self.ps1.decode(getpreferredencoding()))
-            self.stdout_hist += self.ps1
+                self.stdout_hist += self.ps1.encode(getpreferredencoding())
         else:
+            caption = ('prompt_more', self.ps2)
             if py3:
-                caption = ('prompt_more', self.ps2)
+                self.stdout_hist += self.ps2
             else:
-                caption = ('prompt_more', self.ps2.decode(getpreferredencoding()))
-            self.stdout_hist += self.ps2
+                self.stdout_hist += self.ps2.encode(getpreferredencoding())
         self.edit = BPythonEdit(self.config, caption=caption)
 
         urwid.connect_signal(self.edit, 'change', self.on_input_change)
@@ -1064,7 +1070,7 @@ class URWIDRepl(repl.Repl):
             self.tab()
         elif urwid.command_map[event] == 'prev selectable':
             self.tab(True)
-        #else:
+        # else:
         #    self.echo(repr(event))
 
     def tab(self, back=False):
@@ -1119,25 +1125,26 @@ class URWIDRepl(repl.Repl):
         finally:
             self._completion_update_suppressed = False
 
+
 def main(args=None, locals_=None, banner=None):
     translations.init()
 
     # TODO: maybe support displays other than raw_display?
     config, options, exec_args = bpargs.parse(args, (
-            'Urwid options', None, [
-                Option('--twisted', '-T', action='store_true',
-                       help=_('Run twisted reactor.')),
-                Option('--reactor', '-r',
-                       help=_('Select specific reactor (see --help-reactors). '
-                       'Implies --twisted.')),
-                Option('--help-reactors', action='store_true',
-                       help=_('List available reactors for -r.')),
-                Option('--plugin', '-p',
-                       help=_('twistd plugin to run (use twistd for a list). '
-                       'Use "--" to pass further options to the plugin.')),
-                Option('--server', '-s', type='int',
-                       help=_('Port to run an eval server on (forces Twisted).')),
-                ]))
+        'Urwid options', None, [
+            Option('--twisted', '-T', action='store_true',
+                   help=_('Run twisted reactor.')),
+            Option('--reactor', '-r',
+                   help=_('Select specific reactor (see --help-reactors). '
+                          'Implies --twisted.')),
+            Option('--help-reactors', action='store_true',
+                   help=_('List available reactors for -r.')),
+            Option('--plugin', '-p',
+                   help=_('twistd plugin to run (use twistd for a list). '
+                          'Use "--" to pass further options to the plugin.')),
+            Option('--server', '-s', type='int',
+                   help=_('Port to run an eval server on (forces Twisted).')),
+        ]))
 
     if options.help_reactors:
         try:
@@ -1147,7 +1154,7 @@ def main(args=None, locals_=None, banner=None):
                 print('    %-4s\t%s' % (r.shortName, r.description))
         except ImportError:
             sys.stderr.write('No reactors are available. Please install '
-                'twisted for reactor support.\n')
+                             'twisted for reactor support.\n')
         return
 
     palette = [
@@ -1155,8 +1162,8 @@ def main(args=None, locals_=None, banner=None):
          'bold' if color.isupper() else 'default')
         for name, color in iteritems(config.color_scheme)]
     palette.extend([
-            ('bold ' + name, color + ',bold', background, monochrome)
-            for name, color, background, monochrome in palette])
+        ('bold ' + name, color + ',bold', background, monochrome)
+        for name, color, background, monochrome in palette])
 
     if options.server or options.plugin:
         options.twisted = True
@@ -1166,7 +1173,7 @@ def main(args=None, locals_=None, banner=None):
             from twisted.application import reactors
         except ImportError:
             sys.stderr.write('No reactors are available. Please install '
-                'twisted for reactor support.\n')
+                             'twisted for reactor support.\n')
             return
         try:
             # XXX why does this not just return the reactor it installed?
@@ -1175,7 +1182,7 @@ def main(args=None, locals_=None, banner=None):
                 from twisted.internet import reactor
         except reactors.NoSuchReactor:
             sys.stderr.write('Reactor %s does not exist\n' % (
-                    options.reactor,))
+                options.reactor,))
             return
         event_loop = TwistedEventLoop(reactor)
     elif options.twisted:
@@ -1183,7 +1190,7 @@ def main(args=None, locals_=None, banner=None):
             from twisted.internet import reactor
         except ImportError:
             sys.stderr.write('No reactors are available. Please install '
-                'twisted for reactor support.\n')
+                             'twisted for reactor support.\n')
             return
         event_loop = TwistedEventLoop(reactor)
     else:
@@ -1192,18 +1199,14 @@ def main(args=None, locals_=None, banner=None):
         event_loop = None
     # TODO: there is also a glib event loop. Do we want that one?
 
-    # __main__ construction from bpython.cli
-    if locals_ is None:
-        main_mod = sys.modules['__main__'] = ModuleType('__main__')
-        locals_ = main_mod.__dict__
-
+    extend_locals = {}
     if options.plugin:
         try:
             from twisted import plugin
             from twisted.application import service
         except ImportError:
             sys.stderr.write('No twisted plugins are available. Please install '
-                'twisted for twisted plugin support.\n')
+                             'twisted for twisted plugin support.\n')
             return
 
         for plug in plugin.getPlugins(service.IServiceMaker):
@@ -1215,10 +1218,12 @@ def main(args=None, locals_=None, banner=None):
         plugopts = plug.options()
         plugopts.parseOptions(exec_args)
         serv = plug.makeService(plugopts)
-        locals_['service'] = serv
+        extend_locals['service'] = serv
         reactor.callWhenRunning(serv.startService)
         exec_args = []
     interpreter = repl.Interpreter(locals_, locale.getpreferredencoding())
+    # TODO: replace with something less hack-ish
+    interpreter.locals.update(extend_locals)
 
     # This nabs sys.stdin/out via urwid.MainLoop
     myrepl = URWIDRepl(event_loop, palette, interpreter, config)
@@ -1245,6 +1250,7 @@ def main(args=None, locals_=None, banner=None):
     # are called before we get around to starting the mainloop
     # (urwid raises an exception if we try to draw to the screen
     # before starting it).
+
     def run_with_screen_before_mainloop():
         try:
             # Currently we just set this to None because I do not
@@ -1257,7 +1263,7 @@ def main(args=None, locals_=None, banner=None):
             # cannot re-enter the reactor. If using urwid's own
             # mainloop we *might* be able to do something similar and
             # re-enter its mainloop.
-            sys.stdin = None #FakeStdin(myrepl)
+            sys.stdin = None  # FakeStdin(myrepl)
             sys.stdout = myrepl
             sys.stderr = myrepl
 
@@ -1322,6 +1328,7 @@ def main(args=None, locals_=None, banner=None):
         sys.stdout.flush()
     return repl.extract_exit_value(myrepl.exit_value)
 
+
 def load_urwid_command_map(config):
     urwid.command_map[key_dispatch[config.up_one_line_key]] = 'cursor up'
     urwid.command_map[key_dispatch[config.down_one_line_key]] = 'cursor down'
@@ -1334,6 +1341,7 @@ def load_urwid_command_map(config):
     urwid.command_map[key_dispatch[config.clear_word_key]] = 'clear word'
     urwid.command_map[key_dispatch[config.clear_line_key]] = 'clear line'
 
+
 """
             'clear_screen': 'C-l',
             'cut_to_buffer': 'C-k',
